# coding: utf-8

"""
    ActivitySmith API

    Send push notifications and Live Activities to your own devices via a single API key.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from activitysmith_openapi.models.stream_content_state import StreamContentState

class TestStreamContentState(unittest.TestCase):
    """StreamContentState unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> StreamContentState:
        """Test StreamContentState
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `StreamContentState`
        """
        model = StreamContentState()
        if include_optional:
            return StreamContentState(
                title = '',
                subtitle = '',
                number_of_steps = 1,
                current_step = 1,
                percentage = 0,
                value = 1.337,
                upper_limit = 1.337,
                type = 'segmented_progress',
                color = 'blue',
                step_color = 'lime',
                step_colors = [
                    'lime'
                    ],
                metrics = [
                    { }
                    ],
                auto_dismiss_seconds = 0,
                auto_dismiss_minutes = 0
            )
        else:
            return StreamContentState(
                title = '',
        )
        """

    def testStreamContentState(self):
        """Test StreamContentState"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
